searchData={"items":[{"type":"behaviour","title":"MCPServer","doc":"Behaviour definition for implementing MCP (Model Context Protocol) servers.\nProvides a standardized interface for handling MCP protocol messages and tools.","ref":"MCPServer.html"},{"type":"behaviour","title":"Example - MCPServer","doc":"defmodule YourApp.MCPServer do\n      use MCPServer\n      require Logger\n\n      @protocol_version \"2024-11-05\"\n\n      @impl true\n      def handle_ping(request_id) do\n        {:ok, %{jsonrpc: \"2.0\", id: request_id, result: \"pong\"}}\n      end\n\n      @impl true\n      def handle_initialize(request_id, params) do\n        Logger.info(\"Client initialization params: #{inspect(params, pretty: true)}\")\n\n        case validate_protocol_version(params[\"protocolVersion\"]) do\n          :ok ->\n            {:ok,\n             %{\n               jsonrpc: \"2.0\",\n               id: request_id,\n               result: %{\n                 protocolVersion: @protocol_version,\n                 capabilities: %{\n                   tools: %{\n                     listChanged: true\n                   }\n                 },\n                 serverInfo: %{\n                   name: \"Your MCP Server\",\n                   version: \"0.1.0\"\n                 }\n               }\n             }}\n\n          {:error, reason} ->\n            {:error, reason}\n        end\n      end\n\n      @impl true\n      def handle_list_tools(request_id, _params) do\n        {:ok,\n         %{\n           jsonrpc: \"2.0\",\n           id: request_id,\n           result: %{\n             tools: [\n               %{\n                 name: \"upcase\",\n                 description: \"Converts text to uppercase\",\n                 inputSchema: %{\n                   type: \"object\",\n                   required: [\"text\"],\n                   properties: %{\n                     text: %{\n                       type: \"string\",\n                       description: \"The text to convert to uppercase\"\n                     }\n                   }\n                 },\n                 outputSchema: %{\n                   type: \"object\",\n                   required: [\"output\"],\n                   properties: %{\n                     output: %{\n                       type: \"string\",\n                       description: \"The uppercase version of the input text\"\n                     }\n                   }\n                 }\n               }\n             ]\n           }\n         }}\n      end\n\n      @impl true\n      def handle_call_tool(request_id, %{\"name\" => \"upcase\", \"arguments\" => %{\"text\" => text}} = params) do\n        Logger.debug(\"Handling upcase tool call with params: #{inspect(params, pretty: true)}\")\n\n        {:ok,\n         %{\n           jsonrpc: \"2.0\",\n           id: request_id,\n           result: %{\n             content: [\n               %{\n                 type: \"text\",\n                 text: String.upcase(text)\n               }\n             ]\n           }\n         }}\n      end\n\n      def handle_call_tool(request_id, %{\"name\" => unknown_tool} = params) do\n        Logger.warning(\"Unknown tool called: #{unknown_tool} with params: #{inspect(params, pretty: true)}\")\n\n        {:error,\n         %{\n           jsonrpc: \"2.0\",\n           id: request_id,\n           error: %{\n             code: -32601,\n             message: \"Method not found\",\n             data: %{\n               name: unknown_tool\n             }\n           }\n         }}\n      end\n\n      # implementations of other calls for resources, prompts, etc.\n    end","ref":"MCPServer.html#module-example"},{"type":"callback","title":"MCPServer.handle_call_tool/2","doc":"","ref":"MCPServer.html#c:handle_call_tool/2"},{"type":"callback","title":"MCPServer.handle_complete/2","doc":"","ref":"MCPServer.html#c:handle_complete/2"},{"type":"callback","title":"MCPServer.handle_get_prompt/2","doc":"","ref":"MCPServer.html#c:handle_get_prompt/2"},{"type":"callback","title":"MCPServer.handle_initialize/2","doc":"","ref":"MCPServer.html#c:handle_initialize/2"},{"type":"callback","title":"MCPServer.handle_list_prompts/2","doc":"","ref":"MCPServer.html#c:handle_list_prompts/2"},{"type":"callback","title":"MCPServer.handle_list_resources/2","doc":"","ref":"MCPServer.html#c:handle_list_resources/2"},{"type":"callback","title":"MCPServer.handle_list_tools/2","doc":"","ref":"MCPServer.html#c:handle_list_tools/2"},{"type":"callback","title":"MCPServer.handle_ping/1","doc":"","ref":"MCPServer.html#c:handle_ping/1"},{"type":"callback","title":"MCPServer.handle_read_resource/2","doc":"","ref":"MCPServer.html#c:handle_read_resource/2"},{"type":"module","title":"SSE.ConnectionPlug","doc":"A Plug for handling Server-Sent Events (SSE) connections with MCP protocol support.\n\nQuery params:\n  * `sessionId` - The session ID for the connection\n\nThis plug provides two endpoints:\n  * `/sse` - Establishes the SSE connection\n  * `/message` - Handles JSON-RPC messages for the MCP protocol","ref":"SSE.ConnectionPlug.html"},{"type":"module","title":"Usage in Phoenix Router - SSE.ConnectionPlug","doc":"pipeline :sse do\n      plug :accepts, [\"sse\"]\n    end\n\n    scope \"/\" do\n      pipe_through :sse\n      get \"/sse\", SSE.ConnectionPlug, :call\n\n      pipe_through :api\n      post \"/message\", SSE.ConnectionPlug, :call\n    end","ref":"SSE.ConnectionPlug.html#module-usage-in-phoenix-router"},{"type":"module","title":"Usage in Plug Router - SSE.ConnectionPlug","doc":"forward \"/sse\", to: SSE.ConnectionPlug\n    forward \"/message\", to: SSE.ConnectionPlug","ref":"SSE.ConnectionPlug.html#module-usage-in-plug-router"},{"type":"module","title":"SSE.ConnectionRegistry","doc":"Manages the ETS table for SSE connections.\n\nThis module provides a centralized registry for Server-Sent Events (SSE) connections\nusing an ETS table. It ensures the table is created when the application starts and\nprovides access to the table name for other modules to interact with the connection data.","ref":"SSE.ConnectionRegistry.html"},{"type":"function","title":"SSE.ConnectionRegistry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"SSE.ConnectionRegistry.html#child_spec/1"},{"type":"function","title":"SSE.ConnectionRegistry.table_name/0","doc":"Returns the table name used for storing SSE connections.\n\nThe table name is used by other modules to interact with the ETS table\nthat stores SSE connection information.","ref":"SSE.ConnectionRegistry.html#table_name/0"},{"type":"function","title":"Returns - SSE.ConnectionRegistry.table_name/0","doc":"* `:sse_connections` - The atom representing the ETS table name","ref":"SSE.ConnectionRegistry.html#table_name/0-returns"},{"type":"function","title":"Examples - SSE.ConnectionRegistry.table_name/0","doc":"iex> SSE.ConnectionRegistry.table_name()\n    :sse_connections","ref":"SSE.ConnectionRegistry.html#table_name/0-examples"},{"type":"extras","title":"MCP over SSE","doc":"# MCP over SSE\n\nThis library provides a simple implementation of the Model Context Protocol (MCP) over Server-Sent Events (SSE).\n\nFor more information about the Model Context Protocol, visit:\n[Model Context Protocol Documentation](https://modelcontextprotocol.io/introduction)","ref":"readme.html"},{"type":"extras","title":"Installation - MCP over SSE","doc":"","ref":"readme.html#installation"},{"type":"extras","title":"For Phoenix Applications: - MCP over SSE","doc":"1. Add the required configuration to `config/config.exs`:\n\n```elixir\n# Configure MIME types for SSE\nconfig :mime, :types, %{\n  \"text/event-stream\" => [\"sse\"]\n}\n\n# Configure the MCP Server\nconfig :nws_mcp_server, :mcp_server, MCP.DefaultServer\n# config :nws_mcp_server, :mcp_server, YourApp.YourMCPServer\n```\n\n2. Add to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mcp_sse, \"~> 0.1.0\"}\n  ]\nend\n```\n\n3. Configure your router (`lib/your_app_web/router.ex`):\n\n```elixir\npipeline :sse do\n  plug :accepts, [\"sse\"]\nend\n\nscope \"/\" do\n  pipe_through :sse\n  get \"/sse\", SSE.ConnectionPlug, :call\n  \n  pipe_through :api\n  post \"/message\", SSE.ConnectionPlug, :call\nend\n```","ref":"readme.html#for-phoenix-applications"},{"type":"extras","title":"For Plug Applications with Bandit: - MCP over SSE","doc":"1. Create a new Plug application with supervision:\n```bash\nmix new your_app --sup\n```\n\n2. Add the required configuration to `config/config.exs`:\n```elixir\nimport Config\n\n# Configure MIME types for SSE\nconfig :mime, :types, %{\n  \"text/event-stream\" => [\"sse\"]\n}\n\n# Configure the MCP Server\nconfig :your_app, :mcp_server, YourApp.MCPServer\n```\n\n3. Add dependencies to `mix.exs`:\n```elixir\ndef deps do\n  [\n    {:mcp_sse, \"~> 0.1.0\"},\n    {:plug, \"~> 1.14\"},\n    {:bandit, \"~> 1.2\"}\n  ]\nend\n```\n\n4. Configure your router (`lib/your_app/router.ex`):\n```elixir\ndefmodule YourApp.Router do\n  use Plug.Router\n  \n  plug Plug.Parsers,\n    parsers: [:urlencoded, :json],\n    pass: [\"text/*\"],\n    json_decoder: Jason\n\n  plug :match\n  plug :ensure_session_id\n  plug :dispatch\n\n  # Middleware to ensure session ID exists\n  def ensure_session_id(conn, _opts) do\n    case get_session_id(conn) do\n      nil ->\n        # Generate a new session ID if none exists\n        session_id = generate_session_id()\n        %{conn | query_params: Map.put(conn.query_params, \"sessionId\", session_id)}\n      _session_id ->\n        conn\n    end\n  end\n\n  # Helper to get session ID from query params\n  defp get_session_id(conn) do\n    conn.query_params[\"sessionId\"]\n  end\n\n  # Generate a unique session ID\n  defp generate_session_id do\n    Base.encode16(:crypto.strong_rand_bytes(8), case: :lower)\n  end\n\n  forward \"/sse\", to: SSE.ConnectionPlug\n  forward \"/message\", to: SSE.ConnectionPlug\n\n  match _ do\n    send_resp(conn, 404, \"Not found\")\n  end\nend\n```\n\n5. Set up your application supervision (`lib/your_app/application.ex`):\n```elixir\ndefmodule YourApp.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      {Bandit, plug: YourApp.Router, port: 4000}\n    ]\n\n    opts = [strategy: :one_for_one, name: YourApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"readme.html#for-plug-applications-with-bandit"},{"type":"extras","title":"Session Management - MCP over SSE","doc":"The MCP SSE server requires a session ID for each connection. The router automatically:\n- Uses an existing session ID from query parameters if provided\n- Generates a new session ID if none exists\n- Ensures all requests to `/sse` and `/message` endpoints have a valid session ID","ref":"readme.html#session-management"},{"type":"extras","title":"Configuration Options - MCP over SSE","doc":"The Bandit server can be configured with additional options in your application module:\n\n```elixir\n# Example with custom port and HTTPS\nchildren = [\n  {Bandit,\n    plug: YourApp.Router,\n    port: System.get_env(\"PORT\", \"4000\") |> String.to_integer(),\n    scheme: :https,\n    certfile: \"priv/cert/selfsigned.pem\",\n    keyfile: \"priv/cert/selfsigned_key.pem\"\n  }\n]\n```\n\nThe `use MCPServer` macro provides:\n- Built-in message routing\n- Protocol version validation\n- Default implementations for optional callbacks\n- JSON-RPC error handling\n- Logging\n\nYou only need to implement the required callbacks (`handle_ping/1` and `handle_initialize/2`) and any optional callbacks for features you want to support.","ref":"readme.html#configuration-options"},{"type":"extras","title":"Protocol Documentation - MCP over SSE","doc":"For detailed information about the Model Context Protocol, visit:\n[Model Context Protocol Documentation](https://github.com/cursor-ai/model-context-protocol)","ref":"readme.html#protocol-documentation"},{"type":"extras","title":"Features - MCP over SSE","doc":"- Full MCP server implementation\n- SSE connection management\n- JSON-RPC message handling\n- Tool registration and execution\n- Session management\n- Automatic ping/keepalive\n- Error handling and validation","ref":"readme.html#features"},{"type":"extras","title":"Contributing - MCP over SSE","doc":"...","ref":"readme.html#contributing"},{"type":"extras","title":"Quick Demo - MCP over SSE","doc":"To see the MCP server in action:\n\n1. Start the Phoenix server:\n```bash\nmix phx.server\n```\n\n2. In another terminal, run the demo client script:\n```bash\nelixir examples/mcp_client.exs\n```\n\nThe client script will:\n- Connect to the SSE endpoint\n- Initialize the connection\n- List available tools\n- Call the upcase tool with example input\n- Display the results of each step\n\nThis provides a practical demonstration of the MCP protocol flow and server capabilities.","ref":"readme.html#quick-demo"},{"type":"extras","title":"Other Notes - MCP over SSE","doc":"","ref":"readme.html#other-notes"},{"type":"extras","title":"Example Client Usage - MCP over SSE","doc":"```javascript\n// Connect to SSE endpoint\nconst sse = new EventSource('/sse');\n\n// Handle endpoint message\nsse.addEventListener('endpoint', (e) => {\n  const messageEndpoint = e.data;\n  // Use messageEndpoint for subsequent JSON-RPC requests\n});\n\n// Send initialize request\nfetch('/message?sessionId=YOUR_SESSION_ID', {\n  method: 'POST',\n  headers: { 'Content-Type': 'application/json' },\n  body: JSON.stringify({\n    jsonrpc: '2.0',\n    id: 1,\n    method: 'initialize',\n    params: {\n      protocolVersion: '2024-11-05',\n      capabilities: {}\n    }\n  })\n});\n```","ref":"readme.html#example-client-usage"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}